from audiostack.content.script import Script
from audiostack.content.media import Media


def list_projects():
    from audiostack.content.root_functions import Root
    Root.list_projects()
    
def list_modules():
    from audiostack.content.root_functions import Root
    Root.list_projects()